@@ -50,8 +50,10 @@ static char *opt_metadata;
 static char *opt_timestamp = NULL;
 static char *opt_endoflife;
 static char *opt_endoflife_rebase;
+static char **opt_subsets;
 static char *opt_collection_id = NULL;
 static int opt_token_type = -1;
+static gboolean opt_no_summary_index = FALSE;
 
 static GOptionEntry options[] = {
   { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, N_("One line subject"), N_("SUBJECT") },
@@ -66,6 +68,7 @@ static GOptionEntry options[] = {
   { "exclude", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_exclude, N_("Files to exclude"), N_("PATTERN") },
   { "include", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_include, N_("Excluded files to include"), N_("PATTERN") },
   { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, N_("GPG Homedir to use when looking for keyrings"), N_("HOMEDIR") },
+  { "subset", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_subsets, "Add to a named subset", "SUBSET" },
   { "end-of-life", 0, 0, G_OPTION_ARG_STRING, &opt_endoflife, N_("Mark build as end-of-life"), N_("REASON") },
   { "end-of-life-rebase", 0, 0, G_OPTION_ARG_STRING, &opt_endoflife_rebase, N_("Mark build as end-of-life, to be replaced with the given ID"), N_("ID") },
   { "token-type", 0, 0, G_OPTION_ARG_INT, &opt_token_type, N_("Set type of token needed to install this commit"), N_("VAL") },
@@ -73,6 +76,7 @@ static GOptionEntry options[] = {
   { "collection-id", 0, 0, G_OPTION_ARG_STRING, &opt_collection_id, N_("Collection ID"), "COLLECTION-ID" },
   { "disable-fsync", 0, 0, G_OPTION_ARG_NONE, &opt_disable_fsync, "Do not invoke fsync()", NULL },
   { "disable-sandbox", 0, 0, G_OPTION_ARG_NONE, &opt_disable_sandbox, "Do not sandbox icon validator", NULL },
+  { "no-summary-index", 0, 0, G_OPTION_ARG_NONE, &opt_no_summary_index, N_("Don't generate a summary index"), NULL },
 
   { NULL }
 };
@@ -601,6 +605,32 @@ validate_service_file (GFile      *service_file,
   return TRUE;
 }
 
+static gboolean
+get_subsets (char **subsets, GVariant **out)
+{
+  g_autoptr(GVariantBuilder) builder = g_variant_builder_new (G_VARIANT_TYPE ("as"));
+  gboolean found = FALSE;
+
+  if (subsets == NULL)
+    return FALSE;
+
+  for (int i = 0; subsets[i] != NULL; i++)
+    {
+      const char *subset = subsets[i];
+      if (*subset != 0)
+        {
+          found = TRUE;
+          g_variant_builder_add (builder, "s", subset);
+        }
+    }
+
+  if (!found)
+    return FALSE;
+
+  *out = g_variant_ref_sink (g_variant_builder_end (builder));
+  return TRUE;
+}
+
 static gboolean
 validate_exports (GFile *export, GFile *files, const char *app_id, GError **error)
 {
@@ -779,6 +809,7 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
   CommitData commit_data = {0};
   g_auto(GVariantDict) metadata_dict = FLATPAK_VARIANT_DICT_INITIALIZER;
   g_autoptr(GVariant) metadata_dict_v = NULL;
+  g_autoptr(GVariant) subsets_v = NULL;
   gboolean is_runtime = FALSE;
   gboolean is_extension = FALSE;
   guint64 installed_size = 0, download_size = 0;
@@ -818,7 +849,7 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
       goto out;
     }
 
-  if (!flatpak_is_valid_branch (branch, &my_error))
+  if (!flatpak_is_valid_branch (branch, -1, &my_error))
     {
       flatpak_fail (error, _("'%s' is not a valid branch name: %s"), branch, my_error->message);
       goto out;
@@ -1026,7 +1057,7 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
       g_auto(GStrv) full_ref_parts = g_strsplit (full_branch, "/", 0);
       g_autofree char *rebased_ref = g_build_filename (full_ref_parts[0], opt_endoflife_rebase, full_ref_parts[2], full_ref_parts[3], NULL);
 
-      if (!flatpak_is_valid_name (opt_endoflife_rebase, error))
+      if (!flatpak_is_valid_name (opt_endoflife_rebase, -1, error))
         return glnx_prefix_error (error, "Invalid name in --end-of-life-rebase");
 
       g_variant_dict_insert_value (&metadata_dict, OSTREE_COMMIT_META_KEY_ENDOFLIFE_REBASE,
@@ -1035,7 +1066,11 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
 
   if (opt_token_type >= 0)
     g_variant_dict_insert_value (&metadata_dict, "xa.token-type",
-                                 g_variant_new_int32 (opt_token_type));
+                                 g_variant_new_int32 (GINT32_TO_LE (opt_token_type)));
+
+  /* Skip "" subsets as they mean everything, so no  */
+  if (get_subsets (opt_subsets, &subsets_v))
+    g_variant_dict_insert_value (&metadata_dict, "xa.subsets", subsets_v);
 
   metadata_dict_v = g_variant_ref_sink (g_variant_dict_end (&metadata_dict));
 
@@ -1099,13 +1134,21 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
                                         (opt_timestamp != NULL) ? ts.tv_sec : 0, cancellable, error))
     return FALSE;
 
-  if (!opt_no_update_summary &&
-      !flatpak_repo_update (repo,
-                            (const char **) opt_gpg_key_ids,
-                            opt_gpg_homedir,
-                            cancellable,
-                            error))
-    goto out;
+  if (!opt_no_update_summary)
+    {
+      FlatpakRepoUpdateFlags flags = FLATPAK_REPO_UPDATE_FLAG_NONE;
+
+      if (opt_no_summary_index)
+        flags |= FLATPAK_REPO_UPDATE_FLAG_DISABLE_INDEX;
+
+      g_debug ("Updating summary");
+      if (!flatpak_repo_update (repo, flags,
+                                (const char **) opt_gpg_key_ids,
+                                opt_gpg_homedir,
+                                cancellable,
+                                error))
+        goto out;
+    }
 
   format_size = g_format_size (stats.content_bytes_written);
 
