@@ -103,20 +103,27 @@ remote_state_dir_pair_new (FlatpakDir *dir, FlatpakRemoteState *state)
   return pair;
 }
 
+static char *
+strip_last_element (const char *id,
+                    gsize id_len)
+{
+  while (id_len > 0 &&
+         id[id_len - 1] != '.')
+    id_len--;
+
+  if (id_len > 0)
+    id_len--; /* Remove the dot too */
+
+  return g_strndup (id, id_len);
+}
+
 static gboolean
 ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime, Column *columns, GCancellable *cancellable, GError **error)
 {
   FlatpakTablePrinter *printer;
-  GHashTableIter refs_iter;
-  GHashTableIter iter;
-  gpointer refs_key;
-  gpointer refs_value;
-  gpointer key;
-  gpointer value;
   guint n_keys;
-  g_autofree const char **keys = NULL;
+  g_autofree FlatpakDecomposed **keys = NULL;
   int i, j;
-  g_autoptr(GHashTable) pref_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   FlatpakKinds match_kinds;
   g_autofree char *match_id = NULL;
   g_autofree char *match_arch = NULL;
@@ -150,48 +157,23 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
         need_appstream_data = TRUE;
     }
 
-  g_hash_table_iter_init (&refs_iter, refs_hash);
-  while (g_hash_table_iter_next (&refs_iter, &refs_key, &refs_value))
+  GLNX_HASH_TABLE_FOREACH_KV (refs_hash, GHashTable *, refs, RemoteStateDirPair *, remote_state_dir_pair)
     {
-      GHashTable *refs = refs_key;
-      RemoteStateDirPair *remote_state_dir_pair = refs_value;
       FlatpakDir *dir = remote_state_dir_pair->dir;
       FlatpakRemoteState *state = remote_state_dir_pair->state;
       const char *remote = state->remote_name;
       g_autoptr(AsStore) store = NULL;
-      g_autoptr(GHashTable) names = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+      g_autoptr(GHashTable) pref_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL); /* value owned by refs */
+      g_autoptr(GHashTable) names = g_hash_table_new_full ((GHashFunc)flatpak_decomposed_hash, (GEqualFunc)flatpak_decomposed_equal, (GDestroyNotify)flatpak_decomposed_unref, g_free);
 
-      g_hash_table_iter_init (&iter, refs);
-      while (g_hash_table_iter_next (&iter, &key, &value))
+      GLNX_HASH_TABLE_FOREACH (refs, FlatpakDecomposed *, ref)
         {
-          char *ref = key;
-          char *partial_ref;
-          const char *slash = strchr (ref, '/');
-
-          if (slash == NULL)
-            {
-              g_debug ("Invalid remote ref %s", ref);
-              continue;
-            }
-
-          partial_ref = flatpak_make_valid_id_prefix (slash + 1);
+          char *partial_ref = flatpak_make_valid_id_prefix (flatpak_decomposed_get_pref (ref));
           g_hash_table_insert (pref_hash, partial_ref, ref);
         }
 
-      g_hash_table_iter_init (&iter, refs);
-      while (g_hash_table_iter_next (&iter, &key, &value))
+      GLNX_HASH_TABLE_FOREACH_KV (refs, FlatpakDecomposed *, ref, const char *, checksum)
         {
-          const char *ref = key;
-          const char *checksum = value;
-          g_auto(GStrv) parts = NULL;
-
-          parts = flatpak_decompose_ref (ref, NULL);
-          if (parts == NULL)
-            {
-              g_debug ("Invalid remote ref %s", ref);
-              continue;
-            }
-
           if (opt_only_updates)
             {
               g_autoptr(GBytes) deploy_data = flatpak_dir_get_deploy_data (dir, ref, FLATPAK_DEPLOY_VERSION_ANY, cancellable, NULL);
@@ -206,41 +188,44 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
                 continue;
             }
 
-          if (arches != NULL && !g_strv_contains (arches, parts[2]))
+          if (arches != NULL && !flatpak_decomposed_is_arches (ref, -1, arches))
             continue;
 
-          if (strcmp (parts[0], "runtime") == 0 && !opt_runtime)
+          if (flatpak_decomposed_is_runtime (ref) && !opt_runtime)
             continue;
 
-          if (strcmp (parts[0], "app") == 0 && !opt_app)
+          if (flatpak_decomposed_is_app (ref) && !opt_app)
             continue;
 
           if (!opt_all &&
-              strcmp (parts[0], "runtime") == 0 &&
-              flatpak_id_has_subref_suffix (parts[1]))
+              flatpak_decomposed_is_runtime (ref) &&
+              flatpak_decomposed_id_is_subref (ref))
             {
-              g_autofree char *prefix_partial_ref = NULL;
-              char *last_dot = strrchr (parts[1], '.');
+              g_autoptr(FlatpakDecomposed) parent_ref = NULL;
+              gsize id_len;
+              const char *id = flatpak_decomposed_peek_id (ref, &id_len);
+              g_autofree char *parent_id = strip_last_element (id, id_len);
 
-              *last_dot = 0;
-              prefix_partial_ref = g_strconcat (parts[1], "/", parts[2], "/", parts[3], NULL);
-              *last_dot = '.';
+              parent_ref = flatpak_decomposed_new_from_decomposed (ref, FLATPAK_KINDS_RUNTIME,
+                                                                   parent_id, NULL, NULL, NULL);
 
-              if (g_hash_table_lookup (pref_hash, prefix_partial_ref))
+              if (parent_ref != NULL &&
+                  g_hash_table_lookup (pref_hash, flatpak_decomposed_get_pref (parent_ref)))
                 continue;
             }
 
           if (!opt_all && opt_arch == NULL &&
               /* Hide non-primary arches if the primary arch exists */
-              strcmp (arches[0], parts[2]) != 0)
+              !flatpak_decomposed_is_arch (ref, arches[0]))
             {
-              g_autofree char *alt_arch_ref = g_strconcat (parts[0], "/", parts[1], "/", arches[0], "/", parts[3], NULL);
-              if (g_hash_table_lookup (refs, alt_arch_ref))
+              g_autoptr(FlatpakDecomposed) alt_arch = flatpak_decomposed_new_from_decomposed (ref, 0, NULL, arches[0], NULL, NULL);
+
+              if (alt_arch && g_hash_table_lookup (refs, alt_arch))
                 continue;
             }
 
           if (g_hash_table_lookup (names, ref) == NULL)
-            g_hash_table_insert (names, g_strdup (ref), g_strdup (checksum));
+            g_hash_table_insert (names, flatpak_decomposed_ref (ref), g_strdup (checksum));
         }
 
       if (need_appstream_data)
@@ -254,22 +239,25 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
           flatpak_dir_load_appstream_store (dir, remote, NULL, store, NULL, NULL);
         }
 
-      keys = (const char **) g_hash_table_get_keys_as_array (names, &n_keys);
-      g_qsort_with_data (keys, n_keys, sizeof (char *), (GCompareDataFunc) flatpak_strcmp0_ptr, NULL);
+      keys = (FlatpakDecomposed **) g_hash_table_get_keys_as_array (names, &n_keys);
+      g_qsort_with_data (keys, n_keys, sizeof (char *), (GCompareDataFunc) flatpak_decomposed_strcmp_p, NULL);
 
       for (i = 0; i < n_keys; i++)
         {
-          const char *ref = keys[i];
+          FlatpakDecomposed *ref = keys[i];
+          const char *ref_str = flatpak_decomposed_get_ref (ref);
           guint64 installed_size;
           guint64 download_size;
           g_autofree char *runtime = NULL;
           AsApp *app = NULL;
-          g_auto(GStrv) parts = NULL;
           gboolean has_sparse_cache;
           VarMetadataRef sparse_cache;
+          g_autofree char *id = flatpak_decomposed_dup_id (ref);
+          g_autofree char *arch = flatpak_decomposed_dup_arch (ref);
+          g_autofree char *branch = flatpak_decomposed_dup_branch (ref);
 
           /* The sparse cache is optional */
-          has_sparse_cache = flatpak_remote_state_lookup_sparse_cache (state, ref, &sparse_cache, NULL);
+          has_sparse_cache = flatpak_remote_state_lookup_sparse_cache (state, ref_str, &sparse_cache, NULL);
           if (!opt_all && has_sparse_cache)
             {
               const char *eol = var_metadata_lookup_string (sparse_cache, FLATPAK_SPARSE_CACHE_KEY_ENDOFLINE, NULL);
@@ -279,14 +267,12 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
                 continue;
             }
 
-          parts = flatpak_decompose_ref (ref, NULL);
-
           if (need_cache_data)
             {
               g_autofree char *metadata = NULL;
               g_autoptr(GKeyFile) metakey = NULL;
 
-              if (!flatpak_remote_state_load_data (state, ref,
+              if (!flatpak_remote_state_load_data (state, ref_str,
                                                    &download_size, &installed_size, &metadata,
                                                    error))
                 return FALSE;
@@ -297,7 +283,7 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
             }
 
           if (need_appstream_data)
-            app = as_store_find_app (store, ref);
+            app = as_store_find_app (store, ref_str);
 
           if (app_runtime && runtime)
             {
@@ -313,11 +299,15 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
               if (strcmp (columns[j].name, "name") == 0)
                 {
                   const char *name = NULL;
+                  g_autofree char *readable_id = NULL;
 
                   if (app)
                     name = as_app_get_localized_name (app);
 
-                  flatpak_table_printer_add_column (printer, name ? name : strrchr (parts[1], '.') + 1);
+                  if (name == NULL)
+                    readable_id = flatpak_decomposed_dup_readable_id (ref);
+
+                  flatpak_table_printer_add_column (printer, name ? name : readable_id);
                 }
               else if (strcmp (columns[j].name, "description") == 0)
                 {
@@ -330,13 +320,13 @@ ls_remote (GHashTable *refs_hash, const char **arches, const char *app_runtime,
               else if (strcmp (columns[j].name, "version") == 0)
                 flatpak_table_printer_add_column (printer, app ? as_app_get_version (app) : "");
               else if (strcmp (columns[j].name, "ref") == 0)
-                flatpak_table_printer_add_column (printer, ref);
+                flatpak_table_printer_add_column (printer, ref_str);
               else if (strcmp (columns[j].name, "application") == 0)
-                flatpak_table_printer_add_column (printer, parts[1]);
+                flatpak_table_printer_add_column (printer, id);
               else if (strcmp (columns[j].name, "arch") == 0)
-                flatpak_table_printer_add_column (printer, parts[2]);
+                flatpak_table_printer_add_column (printer, arch);
               else if (strcmp (columns[j].name, "branch") == 0)
-                flatpak_table_printer_add_column (printer, parts[3]);
+                flatpak_table_printer_add_column (printer, branch);
               else if (strcmp (columns[j].name, "origin") == 0)
                 flatpak_table_printer_add_column (printer, remote);
               else if (strcmp (columns[j].name, "commit") == 0)
@@ -425,6 +415,17 @@ flatpak_builtin_remote_ls (int argc, char **argv, GCancellable *cancellable, GEr
 
   has_remote = (argc == 2);
 
+  if (opt_arch != NULL)
+    {
+      if (strcmp (opt_arch, "*") == 0)
+        arches = NULL;
+      else
+        {
+          opt_arches[0] = opt_arch;
+          arches = opt_arches;
+        }
+    }
+
   if (has_remote)
     {
       g_autoptr(FlatpakDir) preferred_dir = NULL;
@@ -442,10 +443,14 @@ flatpak_builtin_remote_ls (int argc, char **argv, GCancellable *cancellable, GEr
             return FALSE;
         }
 
-      state = get_remote_state (preferred_dir, argv[1], opt_cached, opt_sideloaded, cancellable, error);
+      state = get_remote_state (preferred_dir, argv[1], opt_cached, opt_sideloaded, opt_arches[0], NULL, cancellable, error);
       if (state == NULL)
         return FALSE;
 
+      if (arches == NULL &&
+          !ensure_remote_state_all_arches (preferred_dir, state, opt_cached, opt_sideloaded, cancellable, error))
+        return FALSE;
+
       if (!flatpak_dir_list_remote_refs (preferred_dir, state, &refs,
                                          cancellable, error))
         return FALSE;
@@ -477,11 +482,15 @@ flatpak_builtin_remote_ls (int argc, char **argv, GCancellable *cancellable, GEr
               if (flatpak_dir_get_remote_disabled (dir, remote_name))
                 continue;
 
-              state = get_remote_state (dir, remote_name, opt_cached, opt_sideloaded,
+              state = get_remote_state (dir, remote_name, opt_cached, opt_sideloaded, opt_arches[0], NULL,
                                         cancellable, error);
               if (state == NULL)
                 return FALSE;
 
+              if (arches == NULL &&
+                  !ensure_remote_state_all_arches (dir, state, opt_cached, opt_sideloaded, cancellable, error))
+                return FALSE;
+
               if (!flatpak_dir_list_remote_refs (dir, state, &refs,
                                                  cancellable, error))
                 return FALSE;
@@ -492,17 +501,6 @@ flatpak_builtin_remote_ls (int argc, char **argv, GCancellable *cancellable, GEr
         }
     }
 
-  if (opt_arch != NULL)
-    {
-      if (strcmp (opt_arch, "*") == 0)
-        arches = NULL;
-      else
-        {
-          opt_arches[0] = opt_arch;
-          arches = opt_arches;
-        }
-    }
-
   /* show origin by default if listing multiple remotes */
   all_columns[5].def = !has_remote;
 
