@@ -32,6 +32,8 @@
 #include "flatpak-context-private.h"
 #include "flatpak-error.h"
 #include "flatpak-utils-http-private.h"
+#include "flatpak-variant-private.h"
+#include "flatpak-dir-private.h"
 #include <ostree.h>
 #include <json-glib/json-glib.h>
 
@@ -52,6 +54,23 @@
 #define FLATPAK_ANSI_ROW_N "\x1b[%d;1H"
 #define FLATPAK_ANSI_CLEAR "\x1b[0J"
 
+#define FLATPAK_XA_CACHE_VERSION 2
+/* version 1 added extra data download size */
+/* version 2 added ot.ts timestamps (to new format) */
+
+#define FLATPAK_XA_SUMMARY_VERSION 1
+/* version 0/missing is standard ostree summary,
+ * version 1 is compact format with inline cache and no deltas
+ */
+
+/* Thse are key names in the per-ref metadata in the summary */
+#define OSTREE_COMMIT_TIMESTAMP "ostree.commit.timestamp"
+#define OSTREE_COMMIT_TIMESTAMP2 "ot.ts" /* Shorter version of the above */
+
+#define FLATPAK_SUMMARY_DIFF_HEADER "xadf"
+
+#define FLATPAK_SUMMARY_HISTORY_LENGTH_DEFAULT 16
+
 gboolean flatpak_set_tty_echo (gboolean echo);
 void flatpak_get_window_size (int *rows,
                               int *cols);
@@ -143,27 +162,36 @@ GBytes * flatpak_read_stream (GInputStream * in,
                               gboolean null_terminate,
                               GError      **error);
 
+gboolean flatpak_bytes_save (GFile        *dest,
+                             GBytes       *bytes,
+                             GCancellable *cancellable,
+                             GError      **error);
+
 gboolean flatpak_variant_save (GFile        *dest,
                                GVariant     *variant,
                                GCancellable *cancellable,
                                GError      **error);
-gboolean flatpak_variant_bsearch_str (GVariant   *array,
-                                      const char *str,
-                                      int        *out_pos);
 GVariant *flatpak_repo_load_summary (OstreeRepo *repo,
                                      GError    **error);
-char **  flatpak_summary_match_subrefs (GVariant   *summary,
-                                        const char *collection_id,
-                                        const char *ref);
-gboolean flatpak_summary_lookup_ref (GVariant   *summary,
-                                     const char *collection_id,
-                                     const char *ref,
-                                     char      **out_checksum,
-                                     GVariant  **out_variant);
-
-gboolean flatpak_name_matches_one_wildcard_prefix (const char         *string,
-                                                   const char * const *maybe_wildcard_prefixes,
-                                                   gboolean            require_exact_match);
+GVariant *flatpak_repo_load_summary_index (OstreeRepo *repo,
+                                           GError    **error);
+GVariant *flatpak_repo_load_digested_summary (OstreeRepo *repo,
+                                              const char *digest,
+                                              GError    **error);
+GPtrArray *flatpak_summary_match_subrefs (GVariant   *summary,
+                                          const char *collection_id,
+                                          FlatpakDecomposed *ref);
+gboolean flatpak_summary_lookup_ref (GVariant      *summary,
+                                     const char    *collection_id,
+                                     const char    *ref,
+                                     char         **out_checksum,
+                                     VarRefInfoRef *out_info);
+gboolean flatpak_summary_find_ref_map (VarSummaryRef  summary,
+                                       const char    *collection_id,
+                                       VarRefMapRef  *refs_out);
+gboolean flatpak_var_ref_map_lookup_ref (VarRefMapRef   ref_map,
+                                         const char    *ref,
+                                         VarRefInfoRef *out_info);
 
 gboolean flatpak_get_allowed_exports (const char     *source_path,
                                       const char     *app_id,
@@ -172,77 +200,16 @@ gboolean flatpak_get_allowed_exports (const char     *source_path,
                                       char         ***allowed_prefixes_out,
                                       gboolean       *require_exact_match_out);
 
-gboolean flatpak_is_valid_name (const char *string,
-                                GError    **error);
-gboolean flatpak_is_valid_branch (const char *string,
-                                  GError    **error);
-gboolean flatpak_has_name_prefix (const char *string,
-                                  const char *name);
-
-char * flatpak_make_valid_id_prefix (const char *orig_id);
-gboolean flatpak_id_has_subref_suffix (const char *id);
-
-char **flatpak_decompose_ref (const char *ref,
-                              GError    **error);
-
-char * flatpak_filter_glob_to_regexp (const char *glob, GError **error);
-gboolean flatpak_parse_filters (const char *data,
-                                GRegex **allow_refs_out,
-                                GRegex **deny_refs_out,
-                                GError **error);
-gboolean flatpak_filters_allow_ref (GRegex *allow_refs,
-                                    GRegex *deny_refs,
-                                    const char *ref);
-
-FlatpakKinds flatpak_kinds_from_bools (gboolean app,
-                                       gboolean runtime);
-
-gboolean flatpak_split_partial_ref_arg (const char   *partial_ref,
-                                        FlatpakKinds  default_kinds,
-                                        const char   *default_arch,
-                                        const char   *default_branch,
-                                        FlatpakKinds *out_kinds,
-                                        char        **out_id,
-                                        char        **out_arch,
-                                        char        **out_branch,
-                                        GError      **error);
-gboolean flatpak_split_partial_ref_arg_novalidate (const char   *partial_ref,
-                                                   FlatpakKinds  default_kinds,
-                                                   const char   *default_arch,
-                                                   const char   *default_branch,
-                                                   FlatpakKinds *out_kinds,
-                                                   char        **out_id,
-                                                   char        **out_arch,
-                                                   char        **out_branch);
-
-int flatpak_compare_ref (const char *ref1,
-                         const char *ref2);
-
-char * flatpak_compose_ref (gboolean    app,
-                            const char *name,
-                            const char *branch,
-                            const char *arch,
-                            GError    **error);
-
-char * flatpak_build_untyped_ref (const char *runtime,
-                                  const char *branch,
-                                  const char *arch);
-char * flatpak_build_runtime_ref (const char *runtime,
-                                  const char *branch,
-                                  const char *arch);
-char * flatpak_build_app_ref (const char *app,
-                              const char *branch,
-                              const char *arch);
-char * flatpak_find_current_ref (const char   *app_id,
-                                 GCancellable *cancellable,
-                                 GError      **error);
-GFile *flatpak_find_deploy_dir_for_ref (const char   *ref,
-                                        FlatpakDir  **dir_out,
-                                        GCancellable *cancellable,
-                                        GError      **error);
-GFile * flatpak_find_files_dir_for_ref (const char   *ref,
-                                        GCancellable *cancellable,
-                                        GError      **error);
+FlatpakDecomposed *flatpak_find_current_ref (const char   *app_id,
+                                             GCancellable *cancellable,
+                                             GError      **error);
+GFile *flatpak_find_deploy_dir_for_ref (FlatpakDecomposed  *ref,
+                                        FlatpakDir        **dir_out,
+                                        GCancellable       *cancellable,
+                                        GError            **error);
+GFile * flatpak_find_files_dir_for_ref (FlatpakDecomposed *ref,
+                                        GCancellable      *cancellable,
+                                        GError           **error);
 GFile * flatpak_find_unmaintained_extension_dir_if_exists (const char   *name,
                                                            const char   *arch,
                                                            const char   *branch,
@@ -254,6 +221,7 @@ FlatpakDeploy * flatpak_find_deploy_for_ref_in (GPtrArray    *dirs,
                                                 GError      **error);
 FlatpakDeploy * flatpak_find_deploy_for_ref (const char   *ref,
                                              const char   *commit,
+                                             FlatpakDir   *opt_user_dir,
                                              GCancellable *cancellable,
                                              GError      **error);
 char ** flatpak_list_deployed_refs (const char   *type,
@@ -313,6 +281,33 @@ g_key_file_load_from_bytes (GKeyFile     *key_file,
 }
 #endif
 
+#if !GLIB_CHECK_VERSION (2, 54, 0)
+static inline gboolean
+g_ptr_array_find_with_equal_func (GPtrArray     *haystack,
+                                  gconstpointer  needle,
+                                  GEqualFunc     equal_func,
+                                  guint         *index_)
+{
+  guint i;
+
+  g_return_val_if_fail (haystack != NULL, FALSE);
+
+  if (equal_func == NULL)
+    equal_func = g_direct_equal;
+
+  for (i = 0; i < haystack->len; i++)
+    {
+      if (equal_func (g_ptr_array_index (haystack, i), needle))
+        {
+          if (index_ != NULL)
+            *index_ = i;
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+#endif
 
 #if !GLIB_CHECK_VERSION (2, 56, 0)
 GDateTime *flatpak_g_date_time_new_from_iso8601 (const gchar *text,
@@ -447,14 +442,38 @@ gboolean flatpak_repo_set_collection_id (OstreeRepo *repo,
 gboolean flatpak_repo_set_deploy_collection_id (OstreeRepo *repo,
                                                 gboolean    deploy_collection_id,
                                                 GError    **error);
+gboolean flatpak_repo_set_deploy_sideload_collection_id (OstreeRepo *repo,
+                                                         gboolean    deploy_collection_id,
+                                                         GError    **error);
+gboolean flatpak_repo_set_summary_history_length (OstreeRepo *repo,
+                                                  guint       length,
+                                                  GError    **error);
+guint    flatpak_repo_get_summary_history_length (OstreeRepo *repo);
 gboolean flatpak_repo_set_gpg_keys (OstreeRepo *repo,
                                     GBytes     *bytes,
                                     GError    **error);
-gboolean flatpak_repo_update (OstreeRepo   *repo,
-                              const char  **gpg_key_ids,
-                              const char   *gpg_homedir,
-                              GCancellable *cancellable,
-                              GError      **error);
+
+GBytes *flatpak_zlib_compress_bytes   (GBytes  *bytes,
+                                       int      level,
+                                       GError **error);
+GBytes *flatpak_zlib_decompress_bytes (GBytes  *bytes,
+                                       GError **error);
+
+GBytes *flatpak_summary_apply_diff (GBytes *old,
+                                    GBytes *diff,
+                                    GError **error);
+
+typedef enum {
+  FLATPAK_REPO_UPDATE_FLAG_NONE = 0,
+  FLATPAK_REPO_UPDATE_FLAG_DISABLE_INDEX = 1 << 0,
+} FlatpakRepoUpdateFlags;
+
+gboolean flatpak_repo_update (OstreeRepo            *repo,
+                              FlatpakRepoUpdateFlags flags,
+                              const char           **gpg_key_ids,
+                              const char            *gpg_homedir,
+                              GCancellable          *cancellable,
+                              GError               **error);
 gboolean flatpak_repo_collect_sizes (OstreeRepo   *repo,
                                      GFile        *root,
                                      guint64      *installed_size,
@@ -495,17 +514,16 @@ gboolean flatpak_mtree_create_dir (OstreeRepo         *repo,
                                    OstreeMutableTree **dir_out,
                                    GError            **error);
 
-
-GVariant * flatpak_bundle_load (GFile   *file,
-                                char   **commit,
-                                char   **ref,
-                                char   **origin,
-                                char   **runtime_repo,
-                                char   **app_metadata,
-                                guint64 *installed_size,
-                                GBytes **gpg_keys,
-                                char   **collection_id,
-                                GError **error);
+GVariant *flatpak_bundle_load (GFile              *file,
+                               char              **commit,
+                               FlatpakDecomposed **ref,
+                               char              **origin,
+                               char              **runtime_repo,
+                               char              **app_metadata,
+                               guint64            *installed_size,
+                               GBytes            **gpg_keys,
+                               char              **collection_id,
+                               GError            **error);
 
 gboolean flatpak_pull_from_bundle (OstreeRepo   *repo,
                                    GFile        *file,
@@ -525,10 +543,12 @@ char * flatpak_pull_from_oci (OstreeRepo            *repo,
                               FlatpakOciRegistry    *registry,
                               const char            *oci_repository,
                               const char            *digest,
+                              const char            *delta_url,
                               FlatpakOciManifest    *manifest,
                               FlatpakOciImage       *image_config,
                               const char            *remote,
                               const char            *ref,
+                              FlatpakPullFlags       flags,
                               FlatpakOciPullProgress progress_cb,
                               gpointer               progress_data,
                               GCancellable          *cancellable,
@@ -538,7 +558,10 @@ gboolean flatpak_mirror_image_from_oci (FlatpakOciRegistry    *dst_registry,
                                         FlatpakOciRegistry    *registry,
                                         const char            *oci_repository,
                                         const char            *digest,
+                                        const char            *remote,
                                         const char            *ref,
+                                        const char            *delta_url,
+                                        OstreeRepo            *repo,
                                         FlatpakOciPullProgress progress_cb,
                                         gpointer               progress_data,
                                         GCancellable          *cancellable,
@@ -546,18 +569,18 @@ gboolean flatpak_mirror_image_from_oci (FlatpakOciRegistry    *dst_registry,
 
 typedef struct
 {
-  char    *id;
-  char    *installed_id;
-  char    *commit;
-  char    *ref;
-  char    *directory;
-  char    *files_path;
-  char    *subdir_suffix;
-  char    *add_ld_path;
-  char   **merge_dirs;
-  int      priority;
-  gboolean needs_tmpfs;
-  gboolean is_unmaintained;
+  char               *id;
+  char               *installed_id;
+  char               *commit;
+  FlatpakDecomposed *ref;
+  char              *directory;
+  char              *files_path;
+  char              *subdir_suffix;
+  char              *add_ld_path;
+  char             **merge_dirs;
+  int                priority;
+  gboolean           needs_tmpfs;
+  gboolean           is_unmaintained;
 } FlatpakExtension;
 
 void flatpak_extension_free (FlatpakExtension *extension);
@@ -785,6 +808,14 @@ void flatpak_appstream_xml_filter (FlatpakXml *appstream,
                                    GRegex *allow_refs,
                                    GRegex *deny_refs);
 
+char * flatpak_filter_glob_to_regexp (const char *glob, gboolean runtime_only, GError **error);
+gboolean flatpak_parse_filters (const char *data,
+                                GRegex **allow_refs_out,
+                                GRegex **deny_refs_out,
+                                GError **error);
+gboolean flatpak_filters_allow_ref (GRegex *allow_refs,
+                                    GRegex *deny_refs,
+                                    const char *ref);
 
 gboolean flatpak_allocate_tmpdir (int           tmpdir_dfd,
                                   const char   *tmpdir_relpath,
@@ -826,38 +857,6 @@ void flatpak_format_choices (const char **choices,
                              const char  *prompt,
                              ...) G_GNUC_PRINTF (2, 3);
 
-typedef void (*FlatpakProgressCallback)(const char *status,
-                                        guint       progress,
-                                        gboolean    estimating,
-                                        gpointer    user_data);
-
-OstreeAsyncProgress *flatpak_progress_new (FlatpakProgressCallback progress,
-                                           gpointer                progress_data);
-
-#if OSTREE_CHECK_VERSION (2019, 6)
-#define FLATPAK_DO_CHAIN_PROGRESS 1
-#endif
-
-#ifdef FLATPAK_DO_CHAIN_PROGRESS
-void flatpak_chained_progress_finish (OstreeAsyncProgress *progress);
-#endif
-
-static inline void
-flatpak_progress_unchain (OstreeAsyncProgress *chained_progress)
-{
-#ifdef FLATPAK_DO_CHAIN_PROGRESS
-  if (chained_progress != NULL)
-    {
-      flatpak_chained_progress_finish (chained_progress);
-      g_object_unref (chained_progress);
-    }
-#endif
-}
-
-typedef OstreeAsyncProgress FlatpakAsyncProgressChained;
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (FlatpakAsyncProgressChained, flatpak_progress_unchain);
-
-FlatpakAsyncProgressChained *flatpak_progress_chain (OstreeAsyncProgress *progress);
 
 static inline void
 flatpak_ostree_progress_finish (OstreeAsyncProgress *progress)
@@ -880,7 +879,9 @@ gboolean flatpak_check_required_version (const char *ref,
                                          GError    **error);
 
 int flatpak_levenshtein_distance (const char *s,
-                                  const char *t);
+                                  gssize      ls,
+                                  const char *t,
+                                  gssize      lt);
 
 char *   flatpak_dconf_path_for_app_id (const char *app_id);
 gboolean flatpak_dconf_path_is_similar (const char *path1,
@@ -895,6 +896,12 @@ gboolean flatpak_repo_resolve_rev (OstreeRepo    *repo,
                                    GCancellable  *cancellable,
                                    GError       **error);
 
+static inline void
+null_safe_g_ptr_array_unref (gpointer data)
+{
+  g_clear_pointer (&data, g_ptr_array_unref);
+}
+
 #define FLATPAK_MESSAGE_ID "c7b39b1e006b464599465e105b361485"
 
 #endif /* __FLATPAK_UTILS_H__ */
